/* *******************************************************************
   Webserver

   How it all works together:

   page   0.htm         includes the f.css in the header
          1.htm         just another page
   css    f.css         a css stylesheet for all on flash (program) memory
   ***************************************************************** */


// some user variables
int anInteger = 123;
float aFloat = 12.34;
String aString = "abcd";

void serverSetup(){
  server.on("/", handlePage);               // send root page
  server.on("/1.htm", handlePage1);         // send another page
  server.on("/f.css", handleCss);                // a stylesheet
  server.on("/favicon.ico", handle204);          // not available, send 204
  server.begin();
}


// the html output
// Advice: check your generated HTML output if it is valid HTML: https://validator.w3.org
// *** HOME ***  0.htm
void handlePage() {
  String message;

  message =  F("<!DOCTYPE html>\n"
               "<html lang='en'>\n"   // english or any other language code you using for your page content
               "<head>\n"
               "<title>" TXT_BOARDNAME " - Board " );
  message += USE_BOARD;
  message += F("</title>\n"
               "<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\">\n"
               "<meta name=\"viewport\" content=\"width=device-width\">\n"
               "<link rel='stylesheet' type='text/css' href='f.css'>\n"
               "</head>\n");
  message += F("<body>\n");
  message += F("<header>\n<h1>" TXT_BOARDNAME " - Board " );
  message += USE_BOARD;
  message += F("</h1>\n"
               "<nav><p><a href=\"/\">[Home]</a> <a href=\"1.htm\">[More]</a></nav>\n"
               "</header>\n"
               "<main>\n");

  // print whatever you need on the web page:
  message += F("<article>\n"
               "<h2>Values</h2>\n"
               "<p>The integer variable  is currently: ");
  message += anInteger;
  message += F("</p>\n");
  message += F("<p>The float variable is currently: ");
  message += aFloat;   
  message += F("</p>\n");      
  message += F("</article>\n");

  message += F("<article>\n"
               "<h2>Another Topic</h2>\n"
               "<p>The String object doesn't update currently: ");
  message += aString;
  message += F("</p>\n");      
  message += F("</article>\n");
  // end of content
  message += F("</main>\n");
  // a footer line
  message += F("<footer>\n"
               "<p>Running since:");
  message += millis() / 1000;
  message += F(" seconds<p>\n"
               "</footer>\n");

  server.send(200, "text/html", message);
}

// *** Other ***  1.htm
// just some basic examples
void handlePage1() {
  String message;

  message =  F("<!DOCTYPE html>\n"
               "<html lang='en'>\n"   // english or any other language code you using for your page content
               "<head>\n"
               "<title>" TXT_BOARDNAME " - Board " );  message += USE_BOARD;
  message += F("</title>\n"
               "<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\">\n"
               "<meta name=\"viewport\" content=\"width=device-width\">\n"
               "<link rel='stylesheet' type='text/css' href='f.css'>\n"
               "</head>\n");
  message += F("<body>\n");
  message += F("<header>\n<h1>" TXT_BOARDNAME " - Board " );
  message += USE_BOARD;
  message += F("</h1>\n"
               "<nav><p><a href=\"/\">[Home]</a> <a href=\"1.htm\">[More]</a></nav>\n"
               "</header>\n"
               "<main>\n");

  // print whatever you need on the web page:
  message += F("<article>\n"
               "<h2>Comments</h2>\n"
               "<p>This is just a simple Demo of a Webserver running on an ESP32 or ESP8266. "
               "Give your webpage a well definied structure. Put the content in 'main'. </p>"
               "<p>Style your webpage with CSS. Avoid using inline styles. "
               "</p>\n");
  message += F("</article>\n");

  // end of content
  message += F("</main>\n");
  // a footer line
  message += F("<footer>\n"
               "<p>Running since:");
  message += millis() / 1000;
  message += F("<p>\n"
               "</footer>\n");

  server.send(200, "text/html", message);
}

void handleCss() {
  // output of stylesheet
  // this is a straight forward example how to generat a static page from program memory
  String message;
  message = F("*{font-family:sans-serif}\n"
              "body{margin:10px}\n"
              "h1, h2{color:white;background:#1279b1;text-align:center}\n"
              "h1{margin:1px;padding:5px;font-size:1.2em}\n"
              "main{text-align:center}\n"
              "nav{background-color:silver;margin:1px;padding:5px;font-size:0.8em}\n"
              "article{margin:0.2em;padding:0.1em;border-style:solid;border-width:thin;border-color:#C0C0C0;text-align:center;background-color:#E5E5E5}\n"
              "footer p{font-size:0.7em;color:dimgray;background:silver;text-align:center;margin-bottom:5px}\n"
             );
  server.send(200, "text/css", message);
}

// Output a "404 not found" page. It includes the parameters which comes handy for test purposes.
void handleNotFound() {
  Serial.println(F("D124 handleNotFound()"));
  String message;
  message += F("404 - File Not Found\n"
               "URI: ");
  message += server.uri();
  message += F("\nMethod: ");
  message += (server.method() == HTTP_GET) ? "GET" : "POST";
  message += F("\nArguments: ");
  message += server.args();
  message += F("\n");
  for (uint8_t i = 0; i < server.args(); i++) {
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
}

// Output a "204 no content" page. Used for the favicon.ico request (as we don't have any favicon currently)
void handle204() {
  server.send(204);                // this page doesn't send back content
}
//
